"""
Copyright (c) 2015 Red Hat, Inc
All rights reserved.

This software may be modified and distributed under the terms
of the BSD license. See the LICENSE file for details.
"""

from __future__ import print_function, unicode_literals

import os
from dock.constants import YUM_REPOS_DIR, DEFAULT_YUM_REPOFILE_NAME, RELATIVE_REPOS_PATH

try:
    from collections import OrderedDict
except ImportError:
    # Python 2.6
    from ordereddict import OrderedDict
from dock.core import DockerTasker
from dock.inner import DockerBuildWorkflow
from dock.plugin import PreBuildPluginsRunner
from dock.plugins.pre_inject_yum_repo import InjectYumRepoPlugin, alter_yum_commands
from dock.util import ImageName, render_yum_repo
from tests.constants import DOCKERFILE_GIT
from dock.plugins.pre_add_yum_repo_by_url import AddYumRepoByUrlPlugin
from tempfile import NamedTemporaryFile
from collections import namedtuple
import requests
from flexmock import flexmock
import os.path


class X(object):
    pass


def test_yuminject_plugin_notwrapped(tmpdir):
    df = """\
FROM fedora
RUN yum install -y python-django
CMD blabla"""
    tmp_df = os.path.join(str(tmpdir), 'Dockerfile')
    with open(tmp_df, mode="w") as fd:
        fd.write(df)

    tasker = DockerTasker()
    workflow = DockerBuildWorkflow(DOCKERFILE_GIT, "test-image")
    setattr(workflow, 'builder', X)

    metalink = 'https://mirrors.fedoraproject.org/metalink?repo=fedora-$releasever&arch=$basearch'

    workflow.files[os.path.join(YUM_REPOS_DIR, DEFAULT_YUM_REPOFILE_NAME)] = render_yum_repo(OrderedDict(
        (('name', 'my-repo'),
         ('metalink', metalink),
         ('enabled', 1),
         ('gpgcheck', 0)),
    ))

    setattr(workflow.builder, 'image_id', "asd123")
    setattr(workflow.builder, 'df_path', tmp_df)
    setattr(workflow.builder, 'df_dir', str(tmpdir))
    setattr(workflow.builder, 'base_image', ImageName(repo='Fedora', tag='21'))
    setattr(workflow.builder, 'git_dockerfile_path', None)
    setattr(workflow.builder, 'git_path', None)
    runner = PreBuildPluginsRunner(tasker, workflow, [{
        'name': InjectYumRepoPlugin.key,
        'args': {
            "wrap_commands": False
        }
    }])
    runner.run()
    assert InjectYumRepoPlugin.key is not None
    with open(tmp_df, 'r') as fd:
        altered_df = fd.read()
    expected_output = r"""FROM fedora
ADD dock-repos/* '/etc/yum.repos.d/'
RUN yum install -y python-django
RUN rm -f '/etc/yum.repos.d/dock-injected.repo'
CMD blabla"""
    assert expected_output == altered_df


def test_yuminject_plugin_wrapped(tmpdir):
    df = """\
FROM fedora
RUN yum install -y python-django
CMD blabla"""
    tmp_df = os.path.join(str(tmpdir), 'Dockerfile')
    with open(tmp_df, mode="w") as fd:
        fd.write(df)

    tasker = DockerTasker()
    workflow = DockerBuildWorkflow(DOCKERFILE_GIT, "test-image")
    setattr(workflow, 'builder', X)

    metalink = 'https://mirrors.fedoraproject.org/metalink?repo=fedora-$releasever&arch=$basearch'

    workflow.files[os.path.join(YUM_REPOS_DIR, DEFAULT_YUM_REPOFILE_NAME)] = render_yum_repo(OrderedDict(
        (('name', 'my-repo'),
         ('metalink', metalink),
         ('enabled', '1'),
         ('gpgcheck', '0')),
    ))

    setattr(workflow.builder, 'image_id', "asd123")
    setattr(workflow.builder, 'df_path', tmp_df)
    setattr(workflow.builder, 'df_dir', str(tmpdir))
    setattr(workflow.builder, 'base_image', ImageName(repo='Fedora', tag='21'))
    setattr(workflow.builder, 'git_dockerfile_path', None)
    setattr(workflow.builder, 'git_path', None)
    runner = PreBuildPluginsRunner(tasker, workflow, [{
        'name': InjectYumRepoPlugin.key,
        'args': {
            "wrap_commands": True
        }
    }])
    runner.run()
    assert InjectYumRepoPlugin.key is not None
    with open(tmp_df, 'r') as fd:
        altered_df = fd.read()
    expected_output = """FROM fedora
RUN printf "[my-repo]\nname=my-repo\nmetalink=https://mirrors.fedoraproject.org/metalink?repo=fedora-\\$releasever&arch=\\$basearch\nenabled=1\ngpgcheck=0\n" >/etc/yum.repos.d/dock-injected.repo && yum install -y python-django && yum clean all && rm -f /etc/yum.repos.d/dock-injected.repo
CMD blabla"""
    assert expected_output == altered_df


def test_yuminject_multiline_wrapped(tmpdir):
    df = """\
FROM fedora
RUN yum install -y httpd \
                   uwsgi
CMD blabla"""
    tmp_df = os.path.join(str(tmpdir), 'Dockerfile')
    with open(tmp_df, mode="w") as fd:
        fd.write(df)

    tasker = DockerTasker()
    workflow = DockerBuildWorkflow(DOCKERFILE_GIT, "test-image")
    setattr(workflow, 'builder', X)

    metalink = 'https://mirrors.fedoraproject.org/metalink?repo=fedora-$releasever&arch=$basearch'

    workflow.files[os.path.join(YUM_REPOS_DIR, DEFAULT_YUM_REPOFILE_NAME)] = render_yum_repo(OrderedDict(
        (('name', 'my-repo'),
        ('metalink', metalink),
        ('enabled', '1'),
        ('gpgcheck', '0')),
    ))
    setattr(workflow.builder, 'image_id', "asd123")
    setattr(workflow.builder, 'df_path', tmp_df)
    setattr(workflow.builder, 'df_dir', str(tmpdir))
    setattr(workflow.builder, 'base_image', ImageName(repo='Fedora', tag='21'))
    setattr(workflow.builder, 'git_dockerfile_path', None)
    setattr(workflow.builder, 'git_path', None)
    runner = PreBuildPluginsRunner(tasker, workflow,
                                   [{'name': InjectYumRepoPlugin.key, 'args': {
                                       "wrap_commands": True
                                   }}])
    runner.run()
    assert InjectYumRepoPlugin.key is not None
    with open(tmp_df, 'r') as fd:
        altered_df = fd.read()
    try:
        altered_df = altered_df.decode("utf-8")
    except AttributeError:
        pass
    expected_output = """FROM fedora
RUN printf "[my-repo]\nname=my-repo\nmetalink=https://mirrors.fedoraproject.org/metalink?repo=fedora-\\$releasever&arch=\\$basearch\nenabled=1\ngpgcheck=0\n" >/etc/yum.repos.d/dock-injected.repo && yum install -y httpd                    uwsgi && yum clean all && rm -f /etc/yum.repos.d/dock-injected.repo
CMD blabla"""
    assert altered_df == expected_output


def test_yuminject_multiline_notwrapped(tmpdir):
    df = """\
FROM fedora
RUN yum install -y httpd \
                   uwsgi
CMD blabla"""
    tmp_df = os.path.join(str(tmpdir), 'Dockerfile')
    with open(tmp_df, mode="w") as fd:
        fd.write(df)

    tasker = DockerTasker()
    workflow = DockerBuildWorkflow(DOCKERFILE_GIT, "test-image")
    setattr(workflow, 'builder', X)

    metalink = r'https://mirrors.fedoraproject.org/metalink?repo=fedora-$releasever&arch=$basearch'

    workflow.files[os.path.join(YUM_REPOS_DIR, DEFAULT_YUM_REPOFILE_NAME)] = render_yum_repo(OrderedDict(
        (('name', 'my-repo'),
         ('metalink', metalink),
         ('enabled', "1"),
         ('gpgcheck', "0")),
    ))
    setattr(workflow.builder, 'image_id', "asd123")
    setattr(workflow.builder, 'df_path', tmp_df)
    setattr(workflow.builder, 'df_dir', str(tmpdir))
    setattr(workflow.builder, 'base_image', ImageName(repo='Fedora', tag='21'))
    setattr(workflow.builder, 'git_dockerfile_path', None)
    setattr(workflow.builder, 'git_path', None)
    runner = PreBuildPluginsRunner(tasker, workflow,
                                   [{'name': InjectYumRepoPlugin.key, 'args': {
                                       "wrap_commands": False
                                   }}])
    runner.run()
    assert InjectYumRepoPlugin.key is not None
    with open(tmp_df, 'r') as fd:
        altered_df = fd.read()
    try:
        altered_df = altered_df.decode("utf-8")
    except AttributeError:
        pass
    expected_output = r"""FROM fedora
ADD dock-repos/* '/etc/yum.repos.d/'
RUN yum install -y httpd                    uwsgi
RUN rm -f '/etc/yum.repos.d/dock-injected.repo'
CMD blabla"""
    assert altered_df == expected_output


def test_complex_df():
    df = """\
FROM fedora
RUN asd
RUN  yum install x
ENV x=y
RUN yum install \
    x \
    y \
    && something else
CMD asd"""
    wrap_cmd = "RUN test && %(yum_command)s && asd"
    out = alter_yum_commands(df, wrap_cmd)
    expected_output = """\
FROM fedora
RUN asd
RUN test && yum install x && asd
ENV x=y
RUN test && yum install     x     y     && something else && asd
CMD asd"""
    assert out == expected_output


repocontent = '''\
[repo]
name=asd
'''
Dockerfile = namedtuple('Dockerfile', ['lines_before_add',
                                       'lines_before_remove',
                                       'lines_after_remove'])


DOCKERFILES = {
    "no maintainer":
        Dockerfile(["# Simple example with no MAINTAINER line\n",
                    "FROM base\n"],
                   # add goes here
                   [" RUN yum -y update\n"],
                   # remove goes here
            []),

    "no yum":
        Dockerfile(["FROM base\n",
                    "# This time there is a MAINTAINER line\n",
                    "# but it's the last last there is\n",
                    "MAINTAINER Example <example@example.com>\n"],
                   # add goes here
            [],
                   # remove goes here
            []),

    "cmd":
        Dockerfile([" From base\n",
                    "LABEL 'a'='b'\n",
                    "MAINTAINER Example <example@example.com>\n"],
                   # add goes here
                   ["RUN some command\n",
                    "RUN some other command\n",
                    "VOLUME ['/data']\n",
                    "# rm line expected on following line\n"],
                   # remove goes here
                   ["CMD ['/bin/bash']\n"]),

    "entrypoint":
        Dockerfile(["FROM base\n",
                    "MAINTAINER Example <example@example.com\n"],
                   # add goes here
                   ["RUN yum update -y\n",
                    "RUN yum install -y example\n"],
                   # remove goes here
                   ["ENTRYPOINT ['/bin/bash']\n",
                    "CMD ['/bin/ls']\n"]),
    }


def prepare(df_path):
    tasker = DockerTasker()
    workflow = DockerBuildWorkflow(DOCKERFILE_GIT, "test-image")
    setattr(workflow, 'builder', X)

    setattr(workflow.builder, 'image_id', "asd123")
    setattr(workflow.builder, 'df_path', str(df_path))
    setattr(workflow.builder, 'df_dir', os.path.dirname(str(df_path)))
    setattr(workflow.builder, 'base_image', ImageName(repo='Fedora', tag='21'))
    setattr(workflow.builder, 'git_dockerfile_path', None)
    setattr(workflow.builder, 'git_path', None)
    (flexmock(requests.Response, content=repocontent)
     .should_receive('raise_for_status')
     .and_return(None))
    (flexmock(requests, get=lambda *_: requests.Response()))
    return tasker, workflow


def test_no_repourls(tmpdir):
    for df in DOCKERFILES.values():
        with NamedTemporaryFile(mode="w+t",
                                prefix="Dockerfile",
                                dir=str(tmpdir)) as f:
            f.writelines(df.lines_before_add +
                         df.lines_before_remove +
                         df.lines_after_remove)
            f.flush()
            tasker, workflow = prepare(f.name)
            runner = PreBuildPluginsRunner(tasker, workflow, [{
                'name': InjectYumRepoPlugin.key,
            }])
            runner.run()
            assert AddYumRepoByUrlPlugin.key is not None

            f.seek(0)
            # Should be unchanged
            assert f.readlines() == (df.lines_before_add +
                                     df.lines_before_remove +
                                     df.lines_after_remove)


def test_single_repourl(tmpdir):
    for df in DOCKERFILES.values():
        with NamedTemporaryFile(mode="w+t",
                                prefix="Dockerfile",
                                dir=str(tmpdir)) as f:
            f.writelines(df.lines_before_add +
                         df.lines_before_remove +
                         df.lines_after_remove)
            f.flush()
            tasker, workflow = prepare(f.name)
            filename = 'test.repo'
            repo_path = os.path.join(YUM_REPOS_DIR, filename)
            workflow.files[repo_path] = repocontent
            runner = PreBuildPluginsRunner(tasker, workflow, [{
                'name': InjectYumRepoPlugin.key,
                'args': {'wrap_commands': False}}])
            runner.run()

            # Was it written correctly?
            repos_dir = os.path.join(str(tmpdir), RELATIVE_REPOS_PATH)
            repofile = os.path.join(repos_dir, filename)
            with open(repofile, "r") as fp:
                assert fp.read() == repocontent

            # Remove the repos/ directory.
            os.remove(repofile)
            os.rmdir(repos_dir)

            # Examine the Dockerfile.

            f.seek(0)
            newdf = f.readlines()
            before_add = len(df.lines_before_add)
            before_remove = len(df.lines_before_remove)

            # Start of file should be unchanged.
            assert newdf[:before_add] == df.lines_before_add

            # Should see a single add line.
            after_add = before_add + 1
            assert (newdf[before_add:after_add] ==
                    ["ADD %s* '/etc/yum.repos.d/'\n" % RELATIVE_REPOS_PATH])

            # Lines from there up to the remove line should be unchanged.
            before_remove = after_add + len(df.lines_before_remove)
            assert (newdf[after_add:before_remove] ==
                    df.lines_before_remove)

            # There should be a final 'rm'
            remove = newdf[before_remove]
            assert remove == "RUN rm -f '/etc/yum.repos.d/%s'\n" % filename

            # Lines after that should be unchanged.
            after_remove = before_remove + 1
            assert newdf[after_remove:] == df.lines_after_remove


def test_multiple_repourls(tmpdir):
    for df in DOCKERFILES.values():
        with NamedTemporaryFile(mode="w+t",
                                prefix="Dockerfile",
                                dir=str(tmpdir)) as f:
            f.writelines(df.lines_before_add +
                         df.lines_before_remove +
                         df.lines_after_remove)
            f.flush()
            tasker, workflow = prepare(f.name)
            filename1 = 'myrepo.repo'
            filename2 = 'repo-2.repo'
            repo_path1 = os.path.join(YUM_REPOS_DIR, filename1)
            repo_path2 = os.path.join(YUM_REPOS_DIR, filename2)
            workflow.files[repo_path1] = repocontent
            workflow.files[repo_path2] = repocontent
            runner = PreBuildPluginsRunner(tasker, workflow, [{
                'name': InjectYumRepoPlugin.key,
                'args': {'wrap_commands': False}}])
            runner.run()

            # Remove the repos/ directory.
            repos_dir = os.path.join(str(tmpdir), RELATIVE_REPOS_PATH)
            for repofile in [filename1, filename2]:
                os.remove(os.path.join(repos_dir, repofile))

            os.rmdir(repos_dir)

            # Examine the Dockerfile.
            f.seek(0)
            newdf = f.readlines()
            before_add = len(df.lines_before_add)
            before_remove = len(df.lines_before_remove)

            # Start of file should be unchanged.
            assert newdf[:before_add] == df.lines_before_add

            # Should see a single add line.
            after_add = before_add + 1
            assert (newdf[before_add:after_add] ==
                    ["ADD %s* '/etc/yum.repos.d/'\n" % RELATIVE_REPOS_PATH])

            # Lines from there up to the remove line should be unchanged.
            before_remove = after_add + len(df.lines_before_remove)
            assert (newdf[after_add:before_remove] ==
                    df.lines_before_remove)

            # For the 'rm' line, they could be in either order
            remove = newdf[before_remove]
            rmline = "RUN rm -f '/etc/yum.repos.d/%s' '/etc/yum.repos.d/%s'\n"
            assert remove in [rmline % (filename1, filename2),
                              rmline % (filename2, filename1)]

            # Lines after that should be unchanged.
            after_remove = before_remove + 1
            assert newdf[after_remove:] == df.lines_after_remove
